//IOBase.cpp
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h> // e.g. sleep 
#include <termios.h> // struct termios
//#include <fcntl.h>
//#include <sys/param.h>
#include "IOBase.h"

///////////////////TIOBase
TIOBase::TIOBase() {
  pFile = new fstream();
}

TIOBase::~TIOBase() {
  delete pFile; // callls fstream::close
}

char TIOBase::OpenFile(char* acFilename, int aiMode) {
  // The calling function should  take care of closing  pFile, e.g
  // with CloseFile(). But we check if it is open.
#ifdef DEBUG
  cout << " using pFile to open: " << acFilename
       << " rdstate: " << pFile->rdstate() << endl;
#endif
  if(!pFile) { cerr << "pFile does not exist!" << endl; return 1; 
  } else {
    // keep the actual filename for messages
    strcpy(cFilename, acFilename); 
    return OpenFile(pFile,acFilename,aiMode);
  }
}

char TIOBase::OpenFile(fstream *apF, char* acFilename, int aiMode) {
  // The calling function should  take care of closing  apF, e.g
  // with CloseFile(). But we check if it is open.
  // Mode, e.g., ios::in, ios::app, ios::out .
#ifdef DEBUG
  cout << " opening... " << acFilename
       << " rdstate: " << apF->rdstate() << endl;
#endif
  // Clear the file descriptor first.
  apF->clear();
  if(!apF) cerr << "apF does not exist!" << endl;
  //apF->tellg(); // try to access it and wait for error.
#ifdef  DEBUG
  cout << " Check " << acFilename << endl;
#endif
  if(apF->bad()) {
    cerr << "Warning: File bad (old file, maybe): " 
	 << cFilename << " (new file): " << acFilename<<"\n";
    // So we clear and close it.
    apF->clear(); apF->close();
  }
  // open the file. One can add an other parameter for the file-rights
  apF->open(acFilename, aiMode);
  if(!apF->good()) {
    cerr << " File not good : " << acFilename << " !\n";
    apF->clear(); apF->close();
    return 1;
  }
#ifdef DEBUG
  cout << acFilename << " open.\n";
#endif
  return 0;
}

char TIOBase::CloseFile() {
#ifdef DEBUG
  cout << " closing  " << cFilename
       << " rdstate: " << pFile->rdstate() << endl;
#endif
  if(pFile) { 
    strcpy(cFilename,"");
    return CloseFile(pFile); 
  } else { cerr<<"CloseFile: pFile does not exsist"<<endl; return 1; }
}

char TIOBase::CloseFile(fstream *apF) {
  // check file. We use the ios statefalgs.
#ifdef DEBUG
  cout << " closing apF " 
       << " rdstate: " << apF->rdstate() << endl;
#endif
  if(!apF) { cerr<<"Error in CloseFile (apF==NULL)."<<endl; exit(1); }
  iFileRdState = apF->rdstate();
  // Also a EOF should not happen, it indicates that a file is not complete.
  if(!apF->good()) { // && !apF->eof()) { // !apF->fail
    cerr << "File error before closing in " << "\n"; 
    iFileRdState = 1;
  }
  // Clear in case of error and close file
  apF->clear(); apF->close(); 
#ifdef DEBUG
  cout << "\n" << " closed regularly.\n"; 
#endif
  if(iFileRdState) return 1; else return 0;
}

char TIOBase::ReadComment() {
  if(pFile) { return ReadComment(pFile);
  } else { 
    cerr << "Error in ReadComment(). No File " 
	 << cFilename << " to read comment from.\n";
    return 1; }
}

char TIOBase::ReadComment(fstream *apF) {
  // skip comment (normally at the beginning of a file)
  char c;
  if(!apF) {
    cerr << "No File " << cFilename << " to read comment from.\n";
    return 1; }
  *apF >> c;
  while((c == '#' || c == ' ' || c == '\n') && apF->good()) {
#ifdef DEBUG
    cout << c;
#endif    
    if(c == '#')
      if(c == '#') apF->ignore(1000, '\n');
    *apF >> c;
  }
  apF->putback(c);
  if(apF->eof()) { 
    cerr << "EOF while reading comment from " << cFilename << "\n"; 
    return 1; 
  }
  if(apF->fail()) { 
    cerr << "FAIL while reading comment from " << cFilename << "\n"; 
    return 1; 
  }
#ifdef DEBUG 
  cout << "\nComment read.\n";
#endif
  return 0;
}

bool TIOBase::DirContainsFile(bool SubSearch, char *fn) {
  DIR *dir;
  struct dirent *dr;
  dir = opendir(".");
  while((dr=readdir(dir))) {
    if(SubSearch) {
      if(strstr(dr->d_name, fn)) { closedir(dir); return 1; }
    } else
      if(!strcmp(fn, dr->d_name)) { closedir(dir); return 1; }
  }
  closedir(dir);
  return 0;
}

int TIOBase::Keypressed(char &ac) {
  struct termios term, termsave;
  // We are not always in noncanonical mode without local echo, so if
  // a key is pressed "outside" this function (which is likely)
  // it will be echoed.
  // tty_noncanonical(STDIN_FILENO);
  if (tcgetattr(STDIN_FILENO, &termsave) < 0) cerr << "get error" << endl;
  term = termsave;
  term.c_lflag &= ~(ICANON | ECHO ); // noncanonical and no echo
  term.c_cc[VMIN] = 0; // buffer size
  term.c_cc[VTIME] = 0;
  if (tcsetattr(STDIN_FILENO, TCSANOW, &term) < 0) 
    cerr << "set error" << endl;  
  // Check for keypressed
  int iReturnValue = read(STDIN_FILENO, &ac, 1);
  // tty_reset(STDIN_FILENO);
  if (tcsetattr(STDIN_FILENO, TCSANOW, &termsave) < 0) 
    cerr << "set error" << endl;
  // Done. Return nuber of chars read.
  return iReturnValue;
}

void TIOBase::KeyCheck() {
  if((cKey!='c') && (cKey!='C')) {// step
    while(Keypressed(cKey)); // while key down (long button press)
    while(!Keypressed(cKey)); // while key up
  } else Keypressed(cKey);
}

