// Uses Motif.
#include "XCluster.h"

//// Cluster
TCluster::TCluster() {
  Data=NULL; Mean = NULL; MeanHierarchy=NULL; NbClustersHierarchy=NULL; 
  DataTagHierarchy=NULL; DataDistMatrix=NULL; ConVec=NULL;
  NbClusters = 5; DataSize = 50;
  DataAreaX =500; DataAreaY=500;
  maxLoops = 1000;
  WaitSeconds = 0; 
  DisplayOnlyResult = 0;
  BaseStepSize = 0.5;
  Resolution = 1;
  ConCutOffFactor = 1;
  time_t t; srand48((long)time(&t)); // Init random generator.
}

TCluster::~TCluster() { 
  if(Data) delete[] Data;
  DeleteAllExceptData(); 
}

void TCluster::DeleteAllExceptData() {
  //  cout<<"DeleteAllExceptData."<<endl;//ddd
  if(Mean) { delete[] Mean; Mean = NULL; }
  DeleteHierarchy(); 
  Delete_DataDistMatrix(); 
  if(ConVec) { delete[] ConVec; ConVec=NULL; }
}

void TCluster::NewData() {
  if(Data) delete[] Data; 
  if(DataSize) Data = new TData[DataSize];
  else { cout<<"!DataSize"<<endl; }//exit(1); }
  for(unsigned int i=0;i<DataSize;i++) {
    Data[i].tag=0; Data[i].Px=0; Data[i].Py=0;
    Data[i].x=0; Data[i].y=0; Data[i].dx=0; Data[i].dy=0;
    Data[i].dist=0; Data[i].weight=0; 
    Data[i].changed = false; }
}

void TCluster::NewMean() {
  if(Mean) delete[] Mean; Mean=NULL;
  if(NbClusters) Mean = new TMean[NbClusters];
  else { cout<<"!NbClusters"<<endl; }//exit(1); }
  for(unsigned int m=0;m<NbClusters;m++) {
    Mean[m].x=0; Mean[m].y=0; Mean[m].dx=0; Mean[m].dy=0; Mean[m].dist=0;   
    Mean[m].C=0; Mean[m].nN=0;
    Mean[m].changed = false; }
}

char TCluster::LoadData() {
  // Use PatternManger to load Data, but not keep its Data structures.
  strcpy(cTrainPatFilename,"XCluster.dat"); 
  if(LoadTrainPattern()) { cout<<"LoadPattern error!"<<endl; return 1; }
   if((TrainData.NbSeq!=1)||(TrainData.NbVal<2)) {
     cerr<<"LoadData error!"<<endl; exit(1); }
   // Copy the one sequence.
   DataSize = TrainData.NbPat[0];
   NewData();
   for(unsigned int i=0;i<DataSize;i++) {
     Data[i].x = TrainData.SeqList[0][i][0];
     Data[i].y = TrainData.SeqList[0][i][1];
     Data[i].changed = false; } 
  DeletePattern(TrainData,0); // Delete TrainData.
  //DataSize=2; NewData//ddd
  //Data[0].x=0;Data[0].y=0;Data[1].x=10;Data[1].y=10;//ddd
  // Do not use PatternManger but load by hand.
  // 2 values per point seperated by white space.
  // Get Min Max of data.
  DataMinX=DBL_MAX; DataMinY=DBL_MAX; DataMaxX=DBL_MIN; DataMaxY=DBL_MIN;
  for(unsigned int i=0;i<DataSize;i++) {
    if(Data[i].x<DataMinX) DataMinX=Data[i].x;
    if(Data[i].y<DataMinY) DataMinY=Data[i].y;
    if(Data[i].x>DataMaxX) DataMaxX=Data[i].x;
    if(Data[i].y>DataMaxY) DataMaxY=Data[i].y; }
  InitData();
  return 0;
}

void TCluster::GenerateData(String DataName) {
  // Data has to be renewed after fixing DataSize.
  // Default: Random point positions.
  DataMinX=0; DataMinY=0; DataMaxX=1; DataMaxY=1;
  DataRangeX = DataMaxX-DataMinX; DataRangeY = DataMaxY-DataMinY;
  NewData();
  for(unsigned int i=0;i<DataSize;i++) {
    Data[i].x = drand48()*DataRangeX+DataMinX;
    Data[i].y = drand48()*DataRangeY+DataMinY;
    Data[i].changed=false; 
  }
  InitData();
}

char TCluster::SaveData() {
  if(OpenFile("XCluster.dat", ios::out)) return 1; // Write log file.
  *pFile<<"# "<<DataSize<<"\n\n";// As comment.
  for(unsigned int i=0;i<DataSize;i++) 
    *pFile<<Data[i].x<<" "<<Data[i].y<<"\n";
  *pFile<<"\n";
  return CloseFile();
}

void TCluster::InitData() {
  // Calc scale from DataArea.
  DataRangeX = DataMaxX-DataMinX; DataRangeY = DataMaxY-DataMinY;
  //DataRangeX *= 1.05; DataRangeY *= 1.05; // Get a empty border.
  // MaxDataArea should always be the initial value.
  if(DataAreaX>DataAreaY) DataAreaY=DataAreaX; else DataAreaX=DataAreaY;
  double CommonScale; // The smaller factor.
  if(DataRangeX>DataRangeY) { 
    CommonScale = DataAreaX/DataRangeX;
    DataAreaY = (unsigned int)ceil(CommonScale*DataRangeY);//DataAreaX stays.
  } else {
    CommonScale = DataAreaY/DataRangeY;
    DataAreaX = (unsigned int)ceil(CommonScale*DataRangeX);//DataAreaY stays.
  }
  DataScaleX = CommonScale;  DataScaleY = CommonScale;
  // Init Data point positions for drawing.
  for(unsigned int i=0;i<DataSize;i++) {
    Data[i].Px = (unsigned int)((Data[i].x-DataMinX)*DataScaleX);
    Data[i].Py = (unsigned int)((Data[i].y-DataMinY)*DataScaleY); 
  }
//   cout<<"DataSize:"<<DataSize
//       <<" DataMinX:"<<DataMinX<<" DataMaxX:"<<DataMaxX
//       <<" DataMinY:"<<DataMinY<<" DataMaxY:"<<DataMaxY
//       <<" DataRangeX:"<<DataRangeX<<" DataRangeY:"<<DataRangeY
//       <<" DataAreaX:"<<DataAreaX<<" DataAreaY:"<<DataAreaY
//       <<" DataScaleX:"<<DataScaleX<<" DataScaleY:"<<DataScaleY
//       <<endl;//ddd
}

void TCluster::Init(String AlgoName) {
  //cout<<"Init."<<endl;//ddd
  // Init common variables before individual algo init.
  loop=0; changed=true;
  if(!Data) GenerateData(NULL);
  if(!strcmp(AlgoName,"k_means")) {
    // Random init cluster means (indirectly by random taggging)..
    if(!NbClusters) NbClusters=1; //OR fmod goes wrong.
    NewMean();
    for(unsigned int i=0;i<DataSize;i++) {
      Data[i].tag = (unsigned int)(fmod(lrand48(),NbClusters));
      //cout<<"i:"<<i<<" Data[i].tag:"<<Data[i].tag<<endl;//ddd
      Mean[Data[i].tag].C++; // Count the number of points in Clusters.
    }
    // Cluster means and actualt Data[].dist will be calculated
    // at the beginning of k_means().
  } // End k_means inti.
  if(!strcmp(AlgoName,"Pairing") || !strcmp(AlgoName,"Grouping")
     || !strcmp(AlgoName,"Neting") || !strcmp(AlgoName,"CNeting")
     || !strcmp(AlgoName,"RCNeting") ) { 
    StepSize=BaseStepSize; // Algo Parameters.
    for(unsigned int i=0;i<DataSize;i++) { 
      Data[i].tag=i; // Every point tagged to himself, to his cluster.
      Data[i].changed=false;
    }
    NbClusters=DataSize; // Every point a clusters in the beginning.
    NewMean();
    for(unsigned int m=0;m<NbClusters;m++) { 
      Mean[m].C=1; Mean[m].dist=DBL_MAX; // Dist to next cluster mean.
      Mean[m].changed=true; Mean[m].x=Data[m].x; Mean[m].y=Data[m].y;
    }
  } // End Pairing or Grouping inti.
  if(!strcmp(AlgoName,"Pairing")) { // Here only Pairing (see above).
    InitHierarchy(false,0);// Init the hierarchy, no DataTagHierarchy.
    StoreHierarchy(); // Remember initial cluster step.
  } // End only Pairing inti.
  if(!strcmp(AlgoName,"Grouping")) { // Here only Grouping (see above).
    InitHierarchy(true,0);// Init the hierarchy with  DataTagHierarchy.
    StoreHierarchy(); // Remember initial cluster step.
  } // End only Grouping inti.
  if(!strcmp(AlgoName,"Neting")) { // Here only Neting (see above).
    // Init the hierarchy without DataTagHierarchy, 
    // MaxHierarchies should be (DataSize/2)+1 (2 big groupgs+thier merger).
    InitHierarchy(true, DataSize); // Let's go save.
    StoreHierarchy(); // Remember initial cluster step.
    HierarchyStep=0; // Number of next neighbors considered.
  } // End only Neting inti.
  // Here only CNeting and RCNeting (see above).
  //if(!strcmp(AlgoName,"CNeting") || !strcmp(AlgoName,"RCNeting")) { 
  // All well taged at the start.
  //  for(unsigned int i=0;i<DataSize;i++)Data[i].changed=true;
  //} // End only CNeting and RCNeting inti.
  // Init common variables before individual algo init.
  NbDistinctClusters=NbClusters;  
}

bool TCluster::k_means()  { // Return 0 to continue.
  double distTmp;
  // Actulaize cluster means.
  for(unsigned int m=0;m<NbClusters;m++) { Mean[m].x=0; Mean[m].y=0; }
  for(unsigned int i=0;i<DataSize;i++) { 
    Mean[Data[i].tag].x+=Data[i].x;
    Mean[Data[i].tag].y+=Data[i].y;
  }
  for(unsigned int m=0;m<NbClusters;m++)
    if(Mean[m].C!=0) { Mean[m].x/=Mean[m].C;  Mean[m].y/=Mean[m].C; } 
  // Recalculate Data[].dist.
  for(unsigned int i=0;i<DataSize;i++) { 
    Data[i].dist=DistPointMean(i,Data[i].tag); }
  // Reasign Data.
  changed=false;
  for(unsigned int i=0;i<DataSize;i++) { 
    Data[i].changed=false;
    for(unsigned int m=0;m<NbClusters;m++) {
      if(Data[i].tag!=m) {
	distTmp = DistPointMean(i,m);
	if(distTmp<Data[i].dist) { // If Cluster m is closer..
	  Mean[m].C++;
	  if(Mean[Data[i].tag].C>0) Mean[Data[i].tag].C--; // Should be.
	  Data[i].dist=distTmp; Data[i].tag=m; 
	  Data[i].changed=true; changed=true; 
	}
      }
    }
  }
  if((++loop==maxLoops)|| !changed) return true; else return false;
}

bool TCluster::Pairing() { // Return 0 to continue.
  // Merge groups in one step.
  // Data is never reassigned, it stays with its group.
  // Local, No initial NbClusters needed, can be done hierarchical.
  // Weight the clusters with number of merged clusters inside.
  // ToTry: merge immediately and continue wtih heigher weight (C).
  double TmpDist, MinDist;
  if(NbClusters==1) { //Already at the end.
    // Select the best hierarchical cluster step and re-display it.
    SelectBestHierarchyStep("NbClusterDecrease");
    AdjustStepSize();
    ReasignData();
    return true; 
  }
  changed=false; 
  for(unsigned int m=0;m<NbClusters;m++) { 
    // Until all clusters close enough to at least one other cluster.
    if(Mean[m].changed) {
      if(Mean[m].dist<Resolution) { 
	Mean[m].dx=0; Mean[m].dy=0; 
	Mean[m].changed=false; 	NbDistinctClusters--;
      } else { // Find the nearest cluster.
	changed=true; // Until all cluster.dist=0.
	MinDist=DBL_MAX; Mean[m].nN=m; // Infintly far, but nearest to itself.
	for(unsigned int mm=0;mm<NbClusters;mm++) { 
	  if(mm!=m) { // Not the distance to itself.
	    TmpDist=DistMeanMean(m,mm);
	    if(TmpDist<MinDist) {
	      MinDist=TmpDist; Mean[m].nN=mm; } // Nearest is  nN.
	  }
	}
	Mean[m].dist=MinDist;
	//cout <<i<<":"<<Mean[m].dist<<":"<<"x"<<Mean[m].x<<"y"<<Mean[m].y
	//   <<"dx"<<Mean[m].dx<<"dy"<<Mean[m].dy<<endl;//ddd
	// Everybody moves towards the next guy (the bigger the slower).
	Mean[m].dx = StepSize/Mean[m].C*(Mean[Mean[m].nN].x-Mean[m].x);
	Mean[m].dy = StepSize/Mean[m].C*(Mean[Mean[m].nN].y-Mean[m].y);
      }
    }
  }
  // Update cluster positions.
  for(unsigned int m=0;m<NbClusters;m++) {
    Mean[m].x += Mean[m].dx; Mean[m].y += Mean[m].dy; }
  // Merge cluster when they stopped moving and reassign the Data.
  if(!changed) {
    MergeClusterGroups();
    AdjustStepSize();
    ReasignData();
    StoreHierarchy(); // Remember this hierarchical cluster step.
  }
  if((++loop==maxLoops)|| !changed) return true; else return false;
}

bool TCluster::Grouping() { // Return 0 to continue.
  // Local, No initial NbClusters needed, can be done hierarchical.
  // Weight the clusters with number of merged clusters inside.
  double TmpDist, MinDist;
  if(NbClusters==1) { //Already at the end.
    MeanHierarchy[HierarchyStep-1][0].dist=0; // Set for clearer statisitcs.
    SelectBestHierarchyStep("nNDistSum");
    return true; 
  }
  // Find the nearest cluster.
  for(unsigned int m=0;m<NbClusters;m++) { 
    Mean[m].changed=false; // For the merging afterwards.
    MinDist=DBL_MAX; Mean[m].nN=m; // Infintly far, but nearest to itself.
    for(unsigned int mm=0;mm<NbClusters;mm++) { 
      if(mm!=m) { // Not the distance to itself.
	TmpDist=DistMeanMean(m,mm);
	if(TmpDist<MinDist) { MinDist=TmpDist; Mean[m].nN=mm; }
      }
    }
    MeanHierarchy[HierarchyStep-1][m].dist=MinDist; // Keep for statisitcs.
    //cout <<m<<":"<<Mean[m].dist<<":"<<"x"<<Mean[m].x<<"y"<<Mean[m].y
    // <<" nN->"<<Mean[m].nN<<endl;//ddd
  }
  // Merge cluster groups.
  MergeClusterGroups();
  StoreHierarchy(); // Remember this hierarchical cluster step.
  return true;
}

bool TCluster::Neting() { // Return false to continue.
  // Find net groups for HierarchyStep=number neighbors.
  if(NbClusters==DataSize) { // More inits that take too long for Init();
    Init_DataDistMatrix(); Calc_DataDistMatrix(); Sort_DataDistMatrix(); }
  if(NbClusters==1) { //Already at the end.
    SelectBestHierarchyStep("NbClusterDecrease");
    return true; 
  }
  //HierarchyStep++; // Consider one neighbor more. // Done in StoreHierarchy
  // Form Groups, by broadcasting a messeage.
  Con_DataDistMatrix(); 
  for(unsigned int i=0;i<DataSize;i++) {Data[i].changed=false; Data[i].tag=0;}
  NbDistinctClusters=0;
  for(unsigned int i=0;i<DataSize;i++) // All grouping in this loop.
    if(!Data[i].changed) {//New group.
      AddToCluster(NbDistinctClusters,i); NbDistinctClusters++; }
  cout<<"Broadcasting done."<<endl;
  NbClusters=NbDistinctClusters;
  // Re-init all Mean. Group Data into Mean, like doing a histogram of tags.
  for(unsigned int m=0;m<NbClusters;m++) { 
    Mean[m].C=0; Mean[m].x=0; Mean[m].y=0; }
  for(unsigned int i=0;i<DataSize;i++) {
    Mean[Data[i].tag].C++;
    Mean[Data[i].tag].x+=Data[i].x; Mean[Data[i].tag].y+=Data[i].y; }
  // UpData Mean(s).
  for(unsigned int m=0;m<NbClusters;m++) if(Mean[m].C) { 
    Mean[m].x/=Mean[m].C; Mean[m].y/=Mean[m].C; }
  StoreHierarchy(); // Remember this hierarchical cluster step.
  return true;
}  

bool TCluster::CNeting() { // Continual Neting
  // Find net groups adding one neighbor-connection at a time.
  if(NbClusters==DataSize) { // More inits that take too long for Init();
    Init_DataDistMatrix();Calc_DataDistMatrix();Init_ConVec(); }
  if(NbClusters==1) { //Already at the end.
    if(OpenFile("CNeting.log", ios::out)) return 1; // Write log file.
    *pFile <<0<<" "<<DataSize<<"\n";
    *pFile <<ConVec[0].dist<<" "<<DataSize<<"\n";
    for(unsigned int p=0;p<ConVecPos;p++) { 
      if(p) *pFile <<ConVec[p].dist<<" "<<ConVec[p-1].NbClusters<<"\n";
      *pFile <<ConVec[p].dist<<" "<<ConVec[p].NbClusters<<"\n";
    } CloseFile();
    // Select most stable clustering (longest dist diff with NbCl=const).
    cout <<"Select most stable CNeting clustering:"<<endl;//ddd
    double lastDist=0, actDist, maxDist=0;
    unsigned int maxConVecPos=0, lastNbCl=DataSize;// Add conn.<maxConVecPos.
    for(unsigned int p=0;p<ConVecPos;p++) { // ConVecPos<=ConVecSize .
      actDist=ConVec[p].dist-lastDist; 
      cout<<p<<":lastNbCl="<<lastNbCl<<":dist="<<ConVec[p].dist
	  <<":actDist="<<actDist<<" ";//ddd
      if(actDist>maxDist) { maxDist=actDist; maxConVecPos=p; 
      cout<<" **maxDist="<<maxDist<<"** ";//ddd
      }
      if(ConVec[p].NbClusters<lastNbCl) {
	lastNbCl=ConVec[p].NbClusters; 
	//if(lastDist<ConVec[p].dist)//Check for RCNeting that dist increased.
	lastDist=ConVec[p].dist; 
      }
    }
    cout <<"\nmaxConVecPos:"<<maxConVecPos<<" maxDist:"<<maxDist;//ddd
    if(maxConVecPos)//ddd
      cout<<" Opt. NbClusters:"<<ConVec[maxConVecPos-1].NbClusters<<endl;//ddd
    // Restore best clustering in one big step adding all used connections.
    // As done in Neting.
    for(unsigned int i=0;i<DataSize;i++) { // Re-init DataDistMatrix.con.
      for(unsigned int ii=0;ii<DataSize;ii++){DataDistMatrix[i][ii].Con=0;} }
    //if(maxConVecPos==0) return true; // One Data point one cluster best.
    for(unsigned int p=0;p<maxConVecPos;p++) { // Add connections.
      DataDistMatrix[ConVec[p].SRC][ConVec[p].DST].Con=1;
      DataDistMatrix[ConVec[p].DST][ConVec[p].SRC].Con=1; }
    for(unsigned int i=0;i<DataSize;i++){Data[i].changed=0;Data[i].tag=i;}
    NbDistinctClusters=0;
    for(unsigned int i=0;i<DataSize;i++) // All grouping in this loop.
      if(!Data[i].changed) { // New group.
	AddToCluster(NbDistinctClusters,i); NbDistinctClusters++; }
    NbClusters=NbDistinctClusters;
    // Re-init all Mean. Group Data into Mean, like doing a histogram of tags.
    for(unsigned int m=0;m<NbClusters;m++) { 
      Mean[m].C=0; Mean[m].x=0; Mean[m].y=0; }
    for(unsigned int i=0;i<DataSize;i++) {
      Mean[Data[i].tag].C++;
      Mean[Data[i].tag].x+=Data[i].x; Mean[Data[i].tag].y+=Data[i].y; }
    // UpData Mean(s).
    for(unsigned int m=0;m<NbClusters;m++) if(Mean[m].C) { 
      Mean[m].x/=Mean[m].C; Mean[m].y/=Mean[m].C; }
    return true; 
  }
  // Form Groups, by broadcasting a messeage from where new connection added.
  unsigned int send=ConVec[ConVecPos].SRC,receive=ConVec[ConVecPos].DST,tmpSR;
  // Do not add new connection within  a cluster.
  if(Data[send].tag==Data[receive].tag) {
    ConVec[ConVecPos].NbClusters=NbClusters; // For statisics.
    ConVecPos++; changed=false; return false; }
  // Add new Connection to Con_DataDistMatrix(), all Data.changed=false.
  //cout <<"CNeting add Connection:"<<ConVecPos<<endl;//ddd  
  DataDistMatrix[send][receive].Con=1; DataDistMatrix[receive][send].Con=1;
  if(Data[send].tag>Data[receive].tag) {// Send smaller tag (not important).
    tmpSR=send; send=receive; receive=tmpSR; }// Only not to make holes.
  // Send SRC tag=ClusterNb to Data point DST.
  //cout<<send<<"in";//ddd
  AddToCluster(Data[send].tag,receive);
  //cout<<"Broadcasting done."<<endl;//ddd
  // Re-init all Mean. Group Data into Mean, like doing a histogram of tags.
  for(unsigned int m=0;m<NbClusters;m++) { 
    Mean[m].C=0;  Mean[m].nN=m; Mean[m].x=0; Mean[m].y=0; }
  for(unsigned int i=0;i<DataSize;i++) {
    Mean[Data[i].tag].C++;
    Mean[Data[i].tag].x+=Data[i].x; Mean[Data[i].tag].y+=Data[i].y; }
  FillHolesInMean(); // Fill in holes in Mean.
  ConVec[ConVecPos].NbClusters=NbClusters; // For statistics.
  ConVecPos++; changed=true; return false;
}  

bool TCluster::RCNeting() {
  // Do a data re-init to mark data to speed-up ReSort_ConVec().
  for(unsigned int i=0;i<DataSize;i++) Data[i].changed = false; 
  bool status = CNeting();
  // Resort ConVec with some reguarization.
  if(changed) ReSort_ConVec();
  return status;
}

double TCluster::DistPointPoint(unsigned int i, unsigned int ii) {
  //if((i>=DataSize)||(i>=DataSize))//ddd
  //{cout<<"i:"<<i<<" ii:"<<ii<<endl; exit(1);}//ddd
  //cout<<"i:"<<i<<" ii:"<<ii<<endl;//ddd
  double xdif = Data[i].x-Data[ii].x;
  double ydif = Data[i].y-Data[ii].y;
  double dist = sqrt(xdif*xdif+ydif*ydif);
  return dist;
}

double TCluster::DistPointMean(unsigned int i, unsigned int m) {
  //if((i>=DataSize)||(m>=NbClusters))//ddd
  //{cout<<"i:"<<i<<" m:"<<m<<endl; exit(1);}//ddd
  //cout<<"i:"<<i<<" m:"<<m<<endl;//ddd
  double xdif = Data[i].x-Mean[m].x;
  double ydif = Data[i].y-Mean[m].y;
  double dist = sqrt(xdif*xdif+ydif*ydif);
  return dist;
  // Go for same cluster size.
  //double ko;
  //if(Data[i].tag==m) {
  //  if(Mean[m].C<=1) ko=0.5; else ko = Mean[m].C/(Mean[m].C-1);
  //} else ko = Mean[m].C/(Mean[m].C+1);
  //return (ko*dist);
}

double TCluster::DistMeanMean(unsigned int m, unsigned int mm) {
  //if((m>=NbClusters)||(mm>=NbClusters))//ddd
  // {cout<<"m:"<<m<<" mm:"<<mm<<endl; exit(1);}//ddd
  //cout<<"m:"<<m<<" mm:"<<mm<<endl;//ddd
  double xdif = Mean[m].x-Mean[mm].x;
  double ydif = Mean[m].y-Mean[mm].y;
  return sqrt(xdif*xdif+ydif*ydif);
}

void TCluster::AdjustStepSize() {
  unsigned int SmallesClusterSize=UINT_MAX;
  for(unsigned int m=0;m<NbClusters;m++)
    if(Mean[m].C<SmallesClusterSize) SmallesClusterSize=Mean[m].C;
  StepSize = BaseStepSize*SmallesClusterSize; // Compensate cluster weights.
  //cout <<"StepSize:"<<StepSize<<endl;//ddd
}

void TCluster::ReasignData() {
  // All Data points to the nearest cluster.
  double distTmp;
  for(unsigned int i=0;i<DataSize;i++) { 
    //Data[i].changed=false;
    Data[i].dist = DBL_MAX;
    for(unsigned int m=0;m<NbClusters;m++) {
      distTmp = DistPointMean(i,m);
      if(distTmp<Data[i].dist) { // Cluster m is closer..
	Data[i].dist=distTmp; Data[i].tag=m; 
      }
    }
  }
}

void TCluster::MergeClusterGroups() {
  // Merge cluster by identifying groups and their calc mean.
  // First follow pointers to get closed loops, init a group,
  // make all group menbers' nN point to the smallest index menber,
  // and store the new group cluster here.
  // If a new pointer points into an existing group -> add.
  // One scan.
  // At the start all clusters had changed=false.
  // Unisited cluster and new ones must have changed==false!!(clling func)
  //cout <<"start merging:"<<NbClusters<<endl;//ddd
  NbDistinctClusters=0; // To count number of new groups.
  TMean GroupMean; unsigned int mg; // To step through a group.
  // To temporarily store group member indices.
  unsigned int mGroup[NbClusters], GroupSize;
  for(unsigned int m=0;m<NbClusters;m++) { 
    if(Mean[m].changed) continue; // Skip clusters already visited.
    mg=m; GroupSize=0;
    GroupMean.C=0; GroupMean.x=0; GroupMean.y=0;
    // Step throug group until closed loop is found
    // (group must not be complete).
    do{
      mGroup[GroupSize]=mg; GroupSize++; // Remember members.
      //cout <<mg<<","<<Mean[mg].x<<","<<Mean[mg].y<<","
      //   <<Mean[mg].nN<<" ";//endl;//ddd
      // Add to group mean.
      GroupMean.C+=Mean[mg].C; 
      GroupMean.x+=Mean[mg].C*Mean[mg].x; 
      GroupMean.y+=Mean[mg].C*Mean[mg].y;
      // Mark visited cluster, step to nN and 
      // let nN  point to smallest index group member.
      Mean[mg].changed=true; mg=Mean[mg].nN; Mean[mGroup[GroupSize-1]].nN=m;
    } while(!Mean[mg].changed); // See if nN has been visited.
    // Start a new group (closed loop) or add to existing one (open loop).
    unsigned int mCheck; // Check if closed.
    for(mCheck=0;mCheck<GroupSize;mCheck++) if(mg==mGroup[mCheck]) break;
    if(mCheck<GroupSize) { // New group.
      // Move the new group the position of m, the smallest index.
      Mean[m].x=GroupMean.x; Mean[m].y=GroupMean.y; Mean[m].C=GroupMean.C;
      NbDistinctClusters++; // Count one more new group.
      cout <<"-new->"<<NbDistinctClusters<<":("<<m<<") ";
    } else { // Add to existing group (multiplication with C already in).
      Mean[Mean[mg].nN].x+=GroupMean.x;Mean[Mean[mg].nN].y+=GroupMean.y;
      Mean[Mean[mg].nN].C+=GroupMean.C;
      // Let everybody point the smallest guy.
      for(unsigned int mm=0;mm<GroupSize;mm++) 
	Mean[mGroup[mm]].nN=Mean[mg].nN;
      //cout <<"-add->"<<NbDistinctClusters<<":("<<Mean[mg].nN<<") ";//ddd
    }
//     cout <<":group:";//ddd
//     for(unsigned int mm=0;mm<GroupSize;mm++) //ddd
//       cout <<mGroup[mm]<<"->"<<Mean[mGroup[mm]].nN<<";";//ddd
//     cout<<endl;//ddd
  }
  // Re-tag the Data after grouping to folow cluster.
  for(unsigned int i=0;i<DataSize;i++) Data[i].tag=Mean[Data[i].tag].nN;
  FillHolesInMean(); // Also Update NbClusters.
  //cout <<"merging done:"<<NbClusters<<endl;//ddd
}

void TCluster::FillHolesInMean() {
  // Fill in the holes by deleting clusters that does not point to itself
  // or have no menber. NbClusters gets updated indirectly.
  //cout <<"FillHolesInMean: NbClusters="<<NbClusters<<endl;//ddd
  for(unsigned int m=0;m<NbClusters;m++) { 
    if((Mean[m].nN!=m) || (Mean[m].C==0)) {
      //cout <<"hole:"<<m<<"C="<<Mean[m].C<<"nN="<<Mean[m].nN<<endl;//ddd
      // Find the next good one starting from the other side.
      //for(unsigned int mm=NbClusters-1;mm>m;mm--) {
      for(unsigned int &mm=--NbClusters;mm>m;mm--) { //mm as shorthand
	//cout <<"Try:"<<mm<<"C="<<Mean[mm].C<<"nN="<<Mean[mm].nN<<" ";//ddd
	//NbClusters=mm; // Just temporarily to squeeze this loop.
	if((Mean[mm].nN==mm) && (Mean[mm].C!=0)) {//Copy it down in the hole.
	  Mean[m].C=Mean[mm].C; 
	  Mean[m].x=Mean[mm].x; Mean[m].y=Mean[mm].y;
	  Mean[m].nN=m; Mean[mm].nN=m; // Update nN to re-tag the Data.
	  //cout <<"move:"<<m<<"<-"<<mm<<endl;//ddd
	  break; } } } }
  // Re-tag the Data after filling holes to folow cluster.
  for(unsigned int i=0;i<DataSize;i++) Data[i].tag=Mean[Data[i].tag].nN;
  // Update and re-init the newclusters (see init).
  for(unsigned int m=0;m<NbClusters;m++) {
    // Update group mean.
    Mean[m].x/=Mean[m].C; Mean[m].y/=Mean[m].C;
    Mean[m].dist=DBL_MAX; Mean[m].changed=true;
    //cout <<m<<"C="<<Mean[m].C<<"x="<<Mean[m].x<<"y="<<Mean[m].y
    // <<"nN="<<Mean[m].nN<<endl;//ddd
  }
  //cout <<"Holes filled. NbClusters="<<NbClusters<<endl;//ddd
}

void TCluster::InitHierarchy(bool InitDataTagHierarchy, 
			     unsigned int aMaxHierarchies) {
  // Assuming the number of cluster recduces at least by 1/2
  // (so that NbClusters<2**MaxHierarchies).
  DeleteHierarchy();
  unsigned int NCTmp=NbClusters;
  if(aMaxHierarchies>0) MaxHierarchies=aMaxHierarchies; else {
    MaxHierarchies=0; while(NCTmp>0) { NCTmp>>=1; MaxHierarchies++;} }
  MeanHierarchy = new (TMean*)[MaxHierarchies];
  for(unsigned int s=0;s<MaxHierarchies;s++) MeanHierarchy[s]=NULL;
  NbClustersHierarchy = new unsigned int[MaxHierarchies];
  if(InitDataTagHierarchy) { // Init the DataTagHierarchy.
    DataTagHierarchy = new (unsigned int*)[MaxHierarchies];
    for(unsigned int s=0;s<MaxHierarchies;s++) 
      DataTagHierarchy[s] = new (unsigned int)[DataSize];
  } else DataTagHierarchy=NULL;
  HierarchyStep=0;
}

void TCluster::DeleteHierarchy() {
  if(MeanHierarchy) {
    for(unsigned int s=0;s<MaxHierarchies;s++) delete[] MeanHierarchy[s];
    delete[] MeanHierarchy; MeanHierarchy=NULL;
    delete[] NbClustersHierarchy; NbClustersHierarchy=NULL;
    if(DataTagHierarchy) {    
      for(unsigned int s=0;s<MaxHierarchies;s++) delete[] DataTagHierarchy[s];
      delete[] DataTagHierarchy; DataTagHierarchy=NULL;
    }
  }
}

void TCluster::StoreHierarchy() {
  NbClustersHierarchy[HierarchyStep] = NbClusters;
  if(MeanHierarchy[HierarchyStep]) delete[] MeanHierarchy[HierarchyStep];
  if(NbClusters) MeanHierarchy[HierarchyStep] = new TMean[NbClusters];
  else { cout<<"!NbClusters"<<endl; }//exit(1); }
  for(unsigned int m=0;m<NbClusters;m++) {
    (MeanHierarchy[HierarchyStep])[m].x=Mean[m].x;
    (MeanHierarchy[HierarchyStep])[m].y=Mean[m].y;
    (MeanHierarchy[HierarchyStep])[m].C=Mean[m].C;
  }
  if(DataTagHierarchy) { // Is set NULL in InitHierarchy if not needed.
    for(unsigned int i=0;i<DataSize;i++)
      DataTagHierarchy[HierarchyStep][i] = Data[i].tag;
  }
  HierarchyStep++;
}

void TCluster::RestoreHierarchy(unsigned int step) {
  HierarchyStep=step; NbClusters = NbClustersHierarchy[HierarchyStep];
  //cout <<"RestoreHierarchy:"<<NbClusters<<endl;//ddd
  for(unsigned int m=0;m<NbClusters;m++) {
    Mean[m].x=(MeanHierarchy[HierarchyStep])[m].x;
    Mean[m].y=(MeanHierarchy[HierarchyStep])[m].y;
    Mean[m].C=(MeanHierarchy[HierarchyStep])[m].C;
    Mean[m].changed=true; Mean[m].dist=DBL_MAX; 
  }
  if(DataTagHierarchy) { // Is set NULL in InitHierarchy if not needed.
    for(unsigned int i=0;i<DataSize;i++)
      Data[i].tag = DataTagHierarchy[HierarchyStep][i];
  }
  HierarchyStep++; // To re-continue from this step.
}

void TCluster::SelectBestHierarchyStep(char* Criterum) {
  // Determine most stable; cluster-step, also restore that step.
  unsigned int step=0; 
  if(!strcmp(Criterum,"NbClusterDecrease")) {
    // dNbCl-after/dNbCl-before maximal:
    //cout<<"NbClustersH:"; for(unsigned int s=0;s<HierarchyStep;s++) //ddd
    //cout<<NbClustersHierarchy[s]<<" "; cout<<endl; //ddd
    double NbClustersRatio=0, NbCRTmp;
    for(unsigned int s=1;s<HierarchyStep-1;s++) {
      NbCRTmp=(double)(NbClustersHierarchy[s]-NbClustersHierarchy[s+1])/
		       (NbClustersHierarchy[s-1]-NbClustersHierarchy[s]);
      if(NbClustersRatio<NbCRTmp) { NbClustersRatio=NbCRTmp; step=s; }
      //cout <<"NbCRTmp("<<s<<";"<<NbClustersHierarchy[s]<<"):"
      //   <<NbCRTmp<<" ";//ddd
    }
    //cout <<"NbClustersRatio("<<step<<"):"<<NbClustersRatio<<endl;//ddd
  }
  if(!strcmp(Criterum,"nNDistSum")) {
    // sum(movement)-before/sum(movement)-after is minimal:
    double* nNDistSum = new double[MaxHierarchies];
    double MaxnNDistSum=0;
    for(unsigned int s=0;s<HierarchyStep;s++) {
      // Calculate movement.
      nNDistSum[s]=0;
      for(unsigned int m=0;m<NbClustersHierarchy[s];m++) {
	nNDistSum[s]+=MeanHierarchy[s][m].dist*MeanHierarchy[s][m].C;
      }
      if(!s) nNDistSum[s]=0; 
      else nNDistSum[s]*=(unsigned int) sqrt(NbClustersHierarchy[s]);
      if(nNDistSum[s]>MaxnNDistSum) { MaxnNDistSum=nNDistSum[s]; step=s; }
    }
  }
  RestoreHierarchy(step);
}

void TCluster::Init_DataDistMatrix() {
  if(DataDistMatrix) Delete_DataDistMatrix();
  DataDistMatrix = new (TNeighbor*)[DataSize];
  for(unsigned int i=0;i<DataSize;i++)
    DataDistMatrix[i] = new (TNeighbor)[DataSize];
  for(unsigned int i=0;i<DataSize;i++) { // Init Nb and Con.
    for(unsigned int ii=0;ii<DataSize;ii++) {
      DataDistMatrix[i][ii].Nb=ii; DataDistMatrix[i][ii].Con=0; 
      DataDistMatrix[i][ii].dist=0; } }
}

void TCluster::Delete_DataDistMatrix() {
  if(!DataDistMatrix) return;
  for(unsigned int i=0;i<DataSize;i++) delete[] DataDistMatrix[i];
  delete[] DataDistMatrix; DataDistMatrix=NULL;
}

void TCluster::Calc_DataDistMatrix() {
  for(unsigned int i=0;i<DataSize;i++) {
    DataDistMatrix[i][i].dist=DBL_MAX; // The furthest away to itself.
    for(unsigned int ii=0;ii<i;ii++) {
      DataDistMatrix[i][ii].dist = DistPointPoint(i,ii);
      DataDistMatrix[ii][i].dist = DataDistMatrix[i][ii].dist;
    }
  }
//   cout<<"DataDistMatrix(unsorted):"<<endl;//ddd
//   for(unsigned int i=0;i<DataSize;i++) {//ddd
//     for(unsigned int ii=0;ii<DataSize;ii++) {//ddd
//       cout <<"["<<i<<"]"<<"["<<ii<<"]"<<"("<<DataDistMatrix[i][ii].Nb<<","
// 	   <<DataDistMatrix[i][ii].dist<<")";//ddd
//     } 
//     cout <<endl;//ddd
//   }
}

void TCluster::Sort_DataDistMatrix() {
  for(unsigned int i=0;i<DataSize;i++) // With DBL_MAX dist to onself.
    qsort(DataDistMatrix[i],DataSize,sizeof(TNeighbor), &CompareDist);
//   cout<<"DataDistMatrix(sorted):"<<endl;//ddd
//   for(unsigned int i=0;i<DataSize;i++) {//ddd
//     for(unsigned int ii=0;ii<DataSize;ii++) {//ddd
//       cout <<"["<<i<<"]"<<"["<<ii<<"]"<<"("<<DataDistMatrix[i][ii].Nb<<","
// 	   <<DataDistMatrix[i][ii].dist<<")";//ddd
//     } 
//     cout <<endl;//ddd
//   }
}

void TCluster::Con_DataDistMatrix() {
  // Redo completely for each HierarchyStep.
  //First Calc dist mean and standart deviation to cut of outlier connections.
  ConMean=0; ConStdDev=0, ConErr=0;
  for(unsigned int i=0;i<DataSize;i++) {
    for(unsigned int s=0;s<HierarchyStep;s++) {
      ConMean+=DataDistMatrix[i][s].dist;
      ConStdDev+=DataDistMatrix[i][s].dist*DataDistMatrix[i][s].dist; } }
  ConMean/=DataSize*HierarchyStep;
  ConStdDev=sqrt(ConStdDev/DataSize*HierarchyStep-ConMean*ConMean);
  //ConCutOffDist = ConMean+ConCutOffFactor*ConStdDev; // Mean>0.
  // Calc ConErr for alternative statistics (no std dva, no gauss).
  for(unsigned int i=0;i<DataSize;i++) {
    for(unsigned int s=0;s<HierarchyStep;s++) {
      ConErr+=fabs(ConMean-DataDistMatrix[i][s].dist); } }
  ConErr/=DataSize*HierarchyStep;
  ConCutOffDist = ConMean+ConCutOffFactor*ConErr; // Mean>0.
  //cout<<"\nConMean:"<<ConMean<<" ConStdDev:"<<ConStdDev
  //    <<" ConErr:"<<ConErr<<" ConCutOffDist:"<<ConCutOffDist<<endl;//ddd
  for(unsigned int i=0;i<DataSize;i++) // Reset Con.
    for(unsigned int ii=0;ii<DataSize;ii++) DataDistMatrix[i][ii].Con=0;
  // Set Con if not dist to large (only to true(1)). No self Con.
  for(unsigned int i=0;i<DataSize;i++) {
    // Consider HierarchyStep neighbors.
    //for(unsigned int s=0;s<HierarchyStep;s++){ 
    for(unsigned int s=0;s<DataSize;s++) { // Consider all neighbors.
      if(DataDistMatrix[i][s].dist<ConCutOffDist) {
      DataDistMatrix[i][DataDistMatrix[i][s].Nb].Con=1; // For i.
      DataDistMatrix[DataDistMatrix[i][s].Nb][i].Con=1; } } } // For Nb.
//   cout <<"Con_DataDistMatrix:";//ddd
//   for(unsigned int i=0;i<DataSize;i++) {//ddd
//     cout <<"\n["<<i<<"]";//ddd
//     for(unsigned int ii=0;ii<DataSize;ii++) {//ddd
//       cout <<DataDistMatrix[i][ii].Con<<" "; } } cout <<endl; //ddd
}

void TCluster::Init_ConVec() {
  // Copy all connection from DataDistMatrix.
  ConVecSize=(unsigned int)((DataSize*DataSize-DataSize)/2);
  if(!ConVecSize) { cout <<"!ConVecSize."<<endl; exit(1); }
  if(ConVec) delete[] ConVec; ConVec = new TConVec[ConVecSize]; 
  if(!ConVec) exit(0); ConVecPos=0;
  unsigned int ic=0; // To index ConVec.
  for(unsigned int i=0;i<DataSize;i++) {
    for(unsigned int ii=0;ii<i;ii++) { // Only under diogonal.
      ConVec[ic].SRC = i; ConVec[ic].DST = ii; ConVec[ic].NbClusters=0;
      ConVec[ic].dist = DataDistMatrix[i][ii].dist; ic++; } }
  if(ConVecSize!=ic) { cout<<"ConVecSize!=ic"<<endl; exit(1); }//ddd
  // Sort after dist.
  //for(unsigned int ic=0;ic<ConVecSize;ic++) cout<<ConVec[ic].dist<<" ";//ddd
  qsort(ConVec,ConVecSize,sizeof(TConVec),&CompareDistInConVec);
  //cout<<"ConVec"<<endl;//ddd
  //for(ic=0;ic<ConVecSize;ic++)//ddd
    //cout <<"["<<ic<<"]"<<ConVec[ic].SRC<<"->"<<ConVec[ic].DST
    // <<"="<<ConVec[ic].dist<<" ";//ddd
  //cout<<endl;//ddd
}
 
void TCluster::SwapInConVec(unsigned int DstPos, unsigned int SrcPos) {
  TConVec TmpConVec; 
  memmove(&TmpConVec, &ConVec[DstPos], sizeof(TConVec));
  memmove(&ConVec[DstPos], &ConVec[SrcPos], sizeof(TConVec));
  memmove(&ConVec[SrcPos], &TmpConVec, sizeof(TConVec));
}

void TCluster::ReSort_ConVec() {
  double DstC, SrcC, TmpC;
  unsigned int TMP;
  //cout<<"ReSort_ConVec:"<<endl;//ddd
  // Regularize.
  for(unsigned int ic=ConVecPos;ic<ConVecSize;ic++) {
    if(!Data[ConVec[ic].SRC].changed &&
       !Data[ConVec[ic].DST].changed) continue;
    // Delete connections within one cluster.
    if(ConVec[ic].SRC==ConVec[ic].DST) { 
      SwapInConVec(ic,ConVecSize-1); ConVecSize--; if(ic){ic--; continue;} }
    SrcC = (double)Mean[Data[ConVec[ic].SRC].tag].C;
    DstC = (double)Mean[Data[ConVec[ic].DST].tag].C;
    // Swich connection from big to small clusters.
    if(SrcC<DstC) {
      TMP=ConVec[ic].SRC; ConVec[ic].SRC=ConVec[ic].DST; ConVec[ic].DST=TMP;
      TmpC=SrcC; SrcC=DstC; DstC=TmpC; 
    }
    if(DstC==1) continue;
    //cout<<"org dist:"
    //<<DataDistMatrix[ConVec[ic].SRC][ConVec[ic].DST].dist<<" ";//ddd
    //cout<<SrcC<<","<<DstC<<" ConVec["<<ic<<"]Dist:"
    //<<ConVec[ic].dist<<" -> ";//ddd
    //ConVec[ic].dist = (SrcC+DstC)/(2*SrcC-DstC);
    ConVec[ic].dist = (SrcC+DstC-1)/(SrcC-DstC+1);
    //ConVec[ic].dist = (SrcC+DstC)/(SrcC+1);
    //cout<<"*"<<ConVec[ic].dist<<" ";
    //ConVec[ic].dist *= DataDistMatrix[ConVec[ic].SRC][ConVec[ic].DST].dist;
    ConVec[ic].dist *= DistPointMean(ConVec[ic].DST,Data[ConVec[ic].SRC].tag);
    //cout<<ConVec[ic].dist<<endl;//ddd
  }
  //qsort(&ConVec[ConVecPos],ConVecSize-ConVecPos,
  // sizeof(TConVec),&CompareDistInConVec);
  // Instead of full qsort, just put the next shortest dist to ConVecPos.
  double MinDist=ConVec[ConVecPos].dist; unsigned int MinPos = ConVecPos;
  for(unsigned int ic=ConVecPos+1;ic<ConVecSize;ic++) {
    if(ConVec[ic].dist<MinDist){MinPos=ic; MinDist=ConVec[ic].dist;} }
  if(ConVecPos!=MinPos) SwapInConVec(ConVecPos,MinPos);
}

void TCluster::AddToCluster(unsigned int ClusterNb, unsigned int receiver) {
  //cout <<"["<<ClusterNb<<"]"<<"->"<<receiver<<"in["
  //     <<Data[receiver].tag<<"]:"<<Data[receiver].changed<<" ";//ddd
  // Check if changed for first init of tag check for continual netting.
  if(Data[receiver].changed && (Data[receiver].tag==ClusterNb)) return;
  Data[receiver].changed=true; Data[receiver].tag=ClusterNb; // Mark.
  for(unsigned int i=0;i<DataSize;i++) // Broadcast group messages.
    if(DataDistMatrix[receiver][i].Con) {
      //cout<<receiver<<"in";//ddd
      AddToCluster(ClusterNb,i); 
    }
}

//// TXClusterGUI
TXClusterGUI::TXClusterGUI()
{ 
  workProcId = 0; algo_workProc = run_k_means; 
  AlgoName = new char[64]; strcpy(AlgoName, "k_means");
  ColPix = NULL;
//      static const String 
//        for(int n=0;n<NbColors;n++)
//  	Colors[n] = new String(;
}

TXClusterGUI::~TXClusterGUI() { 
  if(AlgoName) delete[] AlgoName;
  if(ColPix) delete ColPix;
}

void TXClusterGUI::Init(int argc, char** argv) {
  WIDTH=Cluster.DataAreaX;
  HEIGHT=Cluster.DataAreaY+70; // Space for the menubar and algo text.
  pixmapWidth = WIDTH;
  pixmapHeight = HEIGHT;
  //cout<<"WIDTH:"<<WIDTH<<" HEIGHT:"<<HEIGHT<<endl;//ddd
  InitApp(argc, argv);
}

char TXClusterGUI::InitApp(int argc, char** argv) {
  topLevel = XtVaAppInitialize(&app_context, 
			       "XApp",        // application class name
			       NULL, 0,       // command line option list
			       &argc, argv,   // command line args
			       NULL,          // app defaults file
			       XmNwidth, WIDTH,
			       XmNheight, HEIGHT,
			       NULL);         // terminate varargs list
  InitActions();
  mainWindow = XtVaCreateManagedWidget("mainWindow",xmMainWindowWidgetClass,
				       topLevel, NULL);
  // Init drawingArea.
  char translations[] = {
    "<Expose>: draw()\n\
<Configure>: resize()\n\
~Alt <Key>Return: File_Quit()"};
  drawingArea = XtVaCreateManagedWidget
    ("drawingArea", xmDrawingAreaWidgetClass, mainWindow, 
     XmNtranslations, XtParseTranslationTable(translations),
     XmNbackground, WhitePixelOfScreen(XtScreen(mainWindow)), NULL);
  // Menu.
  InitMenu();
  // PopupDialogs.
  InitPopupDialoges();
  XtRealizeWidget(topLevel);
  InitGraphics();
  XtAppMainLoop(app_context);
  return 0;
}

void TXClusterGUI::InitActions() {
  XtActionsRec actions[] = {
    {"draw",draw},
    {"resize",resize},
    {"File_Quit",File_Quit},
    {"File_Start",File_Start},
    {"File_Cont",File_Cont},
    {"File_Stop",File_Stop},
    {"File_Load_Data",File_Load_Data},
    {"File_Save_Data",File_Save_Data},
    {"File_Generate_Data",File_Generate_Data},
    {"Algo_Parameter",Algo_Parameter},
  };
  XtAppAddActions(app_context, actions, XtNumber(actions));
}

void TXClusterGUI::InitMenu() {
  Widget menuBar, fileButton, fileMenu, separator, start, cont, stop, 
    Load_Data, Save_Data, Generate_Data, quit;
  Widget algoButton, algoMenu, parameter; //algoRadioBox, 
  XtTranslations button_translations;
  // register converter for setting tearoff menus from resource files.
  // XmRepTypeInstallTearOffModelConverter();
  // MenuBar
  menuBar = XmCreateMenuBar(mainWindow, "menuBar", NULL, 0);
  XtManageChild(menuBar);
  // CREATE FILE MENU AND CHILDREN
  //create button that will pop up the menu 
  fileButton = 
    XtVaCreateManagedWidget("File",xmCascadeButtonWidgetClass,
			    menuBar, NULL); 
  // create menu (really a Shell widget and RowColumn widget combo) 
  fileMenu = XmCreatePulldownMenu(menuBar, "fileMenu", NULL, 0);
  //create the buttons in the menus
  start = XtVaCreateManagedWidget("Start",xmPushButtonWidgetClass,
				  fileMenu, NULL);
  cont = XtVaCreateManagedWidget("Cont",xmPushButtonWidgetClass,
				  fileMenu, NULL);
  stop = XtVaCreateManagedWidget("Stop",xmPushButtonWidgetClass,
				  fileMenu, NULL);
  separator = XtVaCreateManagedWidget("separator",
				      xmSeparatorGadgetClass, fileMenu, NULL);
  Load_Data = XtVaCreateManagedWidget("Load Data",xmPushButtonWidgetClass,
				      fileMenu, NULL);
  Save_Data = XtVaCreateManagedWidget("Save Data",xmPushButtonWidgetClass,
				      fileMenu, NULL);
  Generate_Data = XtVaCreateManagedWidget("Generate Data",
					  xmPushButtonWidgetClass,
					  fileMenu, NULL);
  separator = XtVaCreateManagedWidget("separator",
				      xmSeparatorGadgetClass, fileMenu, NULL);
  quit = XtVaCreateManagedWidget("Quit", xmPushButtonWidgetClass,
				 fileMenu, NULL);
  // Specify which menu fileButton will pop up.
  XtVaSetValues(fileButton, XmNsubMenuId, fileMenu, NULL);
  // Action (Callback) functions for the buttons.
  //XtAddCallback(start, XmNactivateCallback, File_Start, NULL);
  //XtAddCallback(cont, XmNactivateCallback, File_Cont, NULL);
  //XtAddCallback(quit, XmNactivateCallback, File_Quit, NULL);
  char quit_translations[] = {"<Btn1Up>: File_Quit()"};
  button_translations = XtParseTranslationTable(quit_translations);
  XtOverrideTranslations(quit, button_translations);
  char start_translations[] = {"<Btn1Up>: File_Start()"};
  button_translations = XtParseTranslationTable(start_translations);
  XtOverrideTranslations(start, button_translations);
  char cont_translations[] = {"<Btn1Up>: File_Cont()"};
  button_translations = XtParseTranslationTable(cont_translations);
  XtOverrideTranslations(cont, button_translations);
  char stop_translations[] = {"<Btn1Up>: File_Stop()"};
  button_translations = XtParseTranslationTable(stop_translations);
  XtOverrideTranslations(stop, button_translations);
  char Load_Data_translations[] = {"<Btn1Up>: File_Load_Data()"};
  button_translations = XtParseTranslationTable(Load_Data_translations);
  XtOverrideTranslations(Load_Data, button_translations);
  char Save_Data_translations[] = {"<Btn1Up>: File_Save_Data()"};
  button_translations = XtParseTranslationTable(Save_Data_translations);
  XtOverrideTranslations(Save_Data, button_translations);
  char Generate_Data_translations[] = {"<Btn1Up>: File_Generate_Data()"};
  button_translations = XtParseTranslationTable(Generate_Data_translations);
  XtOverrideTranslations(Generate_Data, button_translations);
  // Algo menu.
  algoButton = XtVaCreateManagedWidget("Algo",xmCascadeButtonWidgetClass,
				       menuBar, NULL);
  algoMenu = XmCreatePulldownMenu(menuBar, "algoMenu", NULL, 0);
  //algoRadioBox = XmCreateRadioBox(algoMenu, "algoRadioBox", NULL, 0);
  //XtVaSetValues(algoRadioBox, NULL);
  //algoRadioBox = XtVaCreateManagedWidget("algoRadioBox",
  //				 xmRadioBoxGadgetClass,
  //				 algoMenu, NULL);
  k_means_TogBut = XtVaCreateManagedWidget
    ("k_means",xmToggleButtonWidgetClass, algoMenu, XmNset, true,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  Pairing_TogBut = XtVaCreateManagedWidget
    ("Pairing",xmToggleButtonWidgetClass, algoMenu,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  Grouping_TogBut = XtVaCreateManagedWidget
    ("Grouping",xmToggleButtonWidgetClass, algoMenu,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  Neting_TogBut = XtVaCreateManagedWidget
    ("Neting",xmToggleButtonWidgetClass, algoMenu,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  CNeting_TogBut = XtVaCreateManagedWidget
    ("CNeting",xmToggleButtonWidgetClass, algoMenu,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  RCNeting_TogBut = XtVaCreateManagedWidget
    ("RCNeting",xmToggleButtonWidgetClass, algoMenu,
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
   random_TogBut = XtVaCreateManagedWidget
    ("random",xmToggleButtonWidgetClass, algoMenu, 
     XmNindicatorType, XmONE_OF_MANY, XmNvisibleWhenOff, true, NULL);
  //XtManageChild(algoRadioBox);
  separator = XtVaCreateManagedWidget("separator",
				      xmSeparatorGadgetClass, algoMenu, NULL);
  parameter = XtVaCreateManagedWidget("Parameter",xmPushButtonWidgetClass,
				      algoMenu, NULL);
  XtVaSetValues(algoButton, XmNsubMenuId, algoMenu, NULL);
  XtAddCallback(k_means_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(Pairing_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(Grouping_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(Neting_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(CNeting_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(RCNeting_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  XtAddCallback(random_TogBut, XmNvalueChangedCallback, Algo_toggle_cbk, 0);
  char parameter_translations[] = {"<Btn1Up>: Algo_Parameter()"};
  button_translations = XtParseTranslationTable(parameter_translations);
  XtOverrideTranslations(parameter, button_translations);

}

void TXClusterGUI::InitGraphics() {
  // Get XLib Display  ID's to draw.
  display = XtDisplay(drawingArea);
  window = XtWindow(drawingArea);
  // Init color.
  cmap = DefaultColormapOfScreen(XtScreen(drawingArea));
  // Graphics context for DrawingArea.
  gc_mask = GCForeground;
  gc_values.foreground = BlackPixelOfScreen(XtScreen(drawingArea));
  gc = XCreateGC(display, window, gc_mask, &gc_values);
  // Attach GC to the DrawingArea's XmNuserData.
  XtVaSetValues(drawingArea, XmNuserData, gc, NULL);
  // Create a pixmap the same size as the drawing area.
  XtVaGetValues(drawingArea, XmNwidth, &pixmapWidth, NULL);
  XtVaGetValues(drawingArea, XmNheight, &pixmapHeight, NULL);
  pixmap = XCreatePixmap
    (display, RootWindowOfScreen(XtScreen(drawingArea)),
     pixmapWidth, pixmapHeight, DefaultDepthOfScreen(XtScreen(drawingArea)));
  // Clear pixmap with white.
  set_Bg_color();
  XFillRectangle(display, pixmap,gc,0,0,pixmapWidth,pixmapHeight);
  set_Fg_color();
  LoadFont("8X13",gc); // "12x24".
}

void TXClusterGUI::InitPopupDialoges() {
  dialog_shell = XtCreatePopupShell
    ("Parameter Dialog", transientShellWidgetClass, topLevel, NULL, 0);
  dialog_box = XtCreateManagedWidget
    ("dialogBox", xmRowColumnWidgetClass, dialog_shell, NULL, 0);
  dialog_label = XtCreateManagedWidget
    ("Number of Clusters:", xmLabelWidgetClass, dialog_box, NULL, 0);
  dialog_text = XtCreateManagedWidget
    ("dialogText", xmTextWidgetClass, dialog_box, NULL, 0);
  dialog_btn = XtCreateManagedWidget
    ("Done", xmPushButtonWidgetClass, dialog_box, NULL, 0);
  XtAddCallback(dialog_btn, XmNactivateCallback, 
		PopdownDialogBox_cbk, NULL);
  XtAddCallback(dialog_text, XmNactivateCallback, 
		PopdownDialogBox_cbk, NULL);
  XtRealizeWidget(dialog_shell);
}

//// Callback and Aktion functions.

// quit button callback function
void TXClusterGUI::File_Quit(Widget w, XEvent *event, 
			   String *args, Cardinal *num_args) {
  exit(0); 
}  

void TXClusterGUI::File_Start(Widget w, XEvent *event, 
			    String *args, Cardinal *num_args) {
  //LoadFont("8X13",gc); // "12x24".
  Cluster.Init(AlgoName);
  draw_algo();
  workProcId = XtAppAddWorkProc(app_context,algo_workProc,NULL);
  //cout << "algo started id: " << workProcId << endl;
}

void TXClusterGUI::File_Cont(Widget w, XEvent *event,
			     String *args, Cardinal *num_args) {
  // If not yet inited or algo changed.
  if(!Cluster.Mean) { Cluster.Init(AlgoName); draw_algo(); } 
  //else cout<<"No init in Cont"<<endl;//ddd 
  workProcId = XtAppAddWorkProc(app_context,algo_workProc,NULL);
  //cout << "algo cont id: " << workProcId << endl;
}

void TXClusterGUI::File_Stop(Widget w, XEvent *event, 
			    String *args, Cardinal *num_args) {
  if(workProcId) { XtRemoveWorkProc(workProcId); workProcId=0; } else return;
  set_Fg_color();
  XDrawString(display,pixmap,gc,5,20,"Stopped",7);
  Fake_draw();  
}

void TXClusterGUI::File_Load_Data(Widget w, XEvent *event, 
			    String *args, Cardinal *num_args) {
  Cluster.DeleteAllExceptData(); // Need old Data parameters here.
  Cluster.LoadData();
  Cluster.Init(AlgoName); // Re-init (Mean,etc).
  draw_algo();
  //set_Fg_color();
  //XDrawString(display,pixmap,gc,5,20,"Load_Data",9); Fake_draw();  
}

void TXClusterGUI::File_Save_Data(Widget w, XEvent *event, 
			    String *args, Cardinal *num_args) {
  Cluster.SaveData();
}

void TXClusterGUI::File_Generate_Data(Widget w, XEvent *event, 
			    String *args, Cardinal *num_args) {
  Cluster.DeleteAllExceptData(); // Need old Data parameters here.
  Cluster.GenerateData(NULL);
  Cluster.Init(AlgoName); // Re-init (Mean,etc).
  draw_algo();
  //set_Fg_color();
  //XDrawString(display,pixmap,gc,5,20,"Generate_Data",9); Fake_draw();  
}

void TXClusterGUI::Algo_toggle_cbk(Widget w, XtPointer client_data, 
				   XtPointer call_data) {
  // Remove (stop) current workProc.
  if(workProcId) { /// Called while an other algo is running,
    XtRemoveWorkProc(workProcId); workProcId=0;
    //set_Fg_color();
    //XDrawString(display,pixmap,gc,5,20,"Algo changed",12);
    //Fake_draw(); 
  }
  // DeleteAllExceptData to force a new init.
  Cluster.DeleteAllExceptData();
  // Get new AlgoName.
  strcpy(AlgoName,"");
  Boolean ButSet;
  XmString compound;
  XtVaGetValues(w, XmNset, &ButSet, NULL);
  XtVaGetValues(w, XmNlabelString, &compound, NULL);
  XmStringGetLtoR(compound, XmFONTLIST_DEFAULT_TAG, &AlgoName);
  XmStringFree(compound);
  //if(!strcmp(AlgoName,"k_means")) cout << "AlgoName=Algo_k_means";//ddd
  //if(!strcmp(AlgoName,"Pairing")) cout << "AlgoName=Algo_Pairing: ";//ddd
  //if(!strcmp(AlgoName,"Grouping")) cout << "AlgoName=Algo_Grouping: ";//ddd
  //if(!strcmp(AlgoName,"Neting")) cout << "AlgoName=Algo_Neting: ";//ddd
  //if(!strcmp(AlgoName,"CNeting")) cout << "AlgoName=Algo_CNeting: ";//ddd
  //if(!strcmp(AlgoName,"RCNeting")) cout << "AlgoName=Algo_RCNeting: ";//ddd
  //if(!strcmp(AlgoName,"random")) cout << "AlgoName=Algo_random: ";//ddd
  //   if(ButSet) cout << "ButSet=true";
  //   cout << endl;
  //
  // Set all the false. 
  XtVaSetValues(k_means_TogBut, XmNset, false, NULL);
  XtVaSetValues(Pairing_TogBut, XmNset, false, NULL);
  XtVaSetValues(Grouping_TogBut, XmNset, false, NULL);
  XtVaSetValues(Neting_TogBut, XmNset, false, NULL);
  XtVaSetValues(CNeting_TogBut, XmNset, false, NULL);
  XtVaSetValues(RCNeting_TogBut, XmNset, false, NULL);
  XtVaSetValues(random_TogBut, XmNset, false, NULL);
  if(!strcmp(AlgoName,"k_means")) { 
    XtVaSetValues(k_means_TogBut, XmNset, true, NULL);
    algo_workProc = run_k_means;
  }
  if(!strcmp(AlgoName,"Pairing")) {
    XtVaSetValues(Pairing_TogBut, XmNset, true, NULL);
    algo_workProc = run_Pairing;
  }
  if(!strcmp(AlgoName,"Grouping")) {
    XtVaSetValues(Grouping_TogBut, XmNset, true, NULL);
    algo_workProc = run_Grouping;
  }
  if(!strcmp(AlgoName,"Neting")) {
    XtVaSetValues(Neting_TogBut, XmNset, true, NULL);
    algo_workProc = run_Neting;
  }
  if(!strcmp(AlgoName,"CNeting")) {
    XtVaSetValues(CNeting_TogBut, XmNset, true, NULL);
    algo_workProc = run_CNeting;
  }
  if(!strcmp(AlgoName,"RCNeting")) {
    XtVaSetValues(RCNeting_TogBut, XmNset, true, NULL);
    algo_workProc = run_RCNeting;
  }
  if(!strcmp(AlgoName,"random")) {
    XtVaSetValues(random_TogBut, XmNset, true, NULL);
    algo_workProc = run_random;
  }
}

void TXClusterGUI::Algo_Parameter(Widget w, XEvent *event, 
				  String *args, Cardinal *num_args) {
  // Calculate position of dialog box as offset from app. shell
  // then pop it up.
  Position x, y, root_x, root_y;
  XtVaGetValues(topLevel, XmNx, &x, XmNy, &y, NULL);
  XtTranslateCoords(topLevel, x, y, &root_x, &root_y);
  XtVaSetValues(dialog_shell, XmNx, root_x+100, XmNy, root_y+100, NULL);
  XtPopup(dialog_shell, XtGrabNone);//XtGrabNonexclusive);////Exclusive);
  char str[5]; sprintf(str,"%d", Cluster.NbClusters);
  //cout << str << " " << Cluster.NbClusters << endl;
  XtVaSetValues(dialog_text, XmNvalue, str, NULL);
  XRaiseWindow(display, XtWindow(dialog_shell));
}

void TXClusterGUI::PopdownDialogBox_cbk(Widget w, XtPointer client_data, 
					XtPointer call_data) {
  // Retrieve input from text widget and set title bar.
  String string;
  XtVaGetValues(dialog_text, XmNvalue, &string, NULL);
  if (strlen(string)>0) {
    Cluster.NbClusters = atoi(string);
    //cout << string << " " << Cluster.NbClusters << endl;
    XFree(string);
  }
  // Pop down dialog box.
  XtPopdown(dialog_shell);
}

void TXClusterGUI::draw_algo() {
  char str[256];
  set_color_pixels();
  set_Bg_color(); // Clear screen.
  XFillRectangle(display, pixmap,gc,0,0,pixmapWidth,pixmapHeight);
  set_Fg_color();
  XDrawLine(display, pixmap,gc,0,
	    Cluster.DataAreaY,Cluster.DataAreaX,Cluster.DataAreaY);
  XDrawLine(display, pixmap,gc,
	    Cluster.DataAreaX,0,Cluster.DataAreaX,Cluster.DataAreaY);
  sprintf(str,"DataSize:%d Range:(x:%.2f;%.2f)(y:%.2f;%.2f)",
	  Cluster.DataSize,Cluster.DataMinX,Cluster.DataMaxX,
	  Cluster.DataMinY,Cluster.DataMaxY);
  XDrawString(display,pixmap,gc,5,pixmapHeight-25,str,strlen(str));
  if((!strcmp(AlgoName,"CNeting") || !strcmp(AlgoName,"RCNeting"))
     && Cluster.ConVec &&(Cluster.ConVecPos)) {
    sprintf(str,"%s  NbClusters:%d  ConVecPos:%d  dist:%-10.2f",AlgoName,
	    Cluster.NbClusters,Cluster.ConVecPos-1,
	    (Cluster.ConVec[Cluster.ConVecPos-1]).dist); }
  else {
    sprintf(str,"%s  loop:%.2d  NbClusters:%d  HierarchyStep:%d",
	    AlgoName,Cluster.loop,
	    Cluster.NbDistinctClusters,Cluster.HierarchyStep-1); }
  XDrawString(display,pixmap,gc,5,pixmapHeight-5,str,strlen(str));
//   for(int i=0;i<Cluster.DataAreaX;i+=10) {//ddd
//     sprintf(str,"%d",i);//ddd
//     XDrawString(display,pixmap,gc,i,i,str,strlen(str)); 
//       XFillArc(display,pixmap,gc,i,i,5,5,0,23040); //360*64=23040//ddd
//   }//ddd
  for(unsigned int i=0;i<Cluster.DataSize;i++) {
    //set_color(Colors[(unsigned int)(fmod(Cluster.Data[i].tag,NbColors))]);
    set_color((long unsigned int)(fmod(Cluster.Data[i].tag,NbColors)));
    //XDrawPoint(display,pixmap,gc,(unsigned int)Cluster.Data[i].x,
    //(unsigned int)Cluster.Data[i].y);
    XDrawArc(display,pixmap,gc,Cluster.Data[i].Px-2,Cluster.Data[i].Py-2,
	     4,4,0,23040); //360*64=23040
  }
  // Draw connections
  if((!strcmp(AlgoName,"CNeting") || !strcmp(AlgoName,"Neting")
      || !strcmp(AlgoName,"RCNeting") )
     && Cluster.DataDistMatrix) {
    for(unsigned int i=0;i<Cluster.DataSize;i++) {
      for(unsigned int ii=0;ii<Cluster.DataSize;ii++) {
	if(Cluster.DataDistMatrix[i][ii].Con) {
	  //set_color(Colors[(unsigned int)
	  //	  (fmod(Cluster.Data[i].tag,NbColors))]);
	  set_color((long unsigned int)(fmod(Cluster.Data[i].tag,NbColors)));
	  XDrawLine(display, pixmap,gc,
		    Cluster.Data[i].Px,Cluster.Data[i].Py,
		    Cluster.Data[ii].Px,Cluster.Data[ii].Py); } } } }
  for(unsigned int m=0;m<Cluster.NbClusters;m++) {
    //set_color(Colors[(unsigned int)(fmod(m,NbColors))]);
    set_color((long unsigned int)(fmod(m,NbColors)));
    XFillRectangle(display,pixmap,gc,
		   (unsigned int)((Cluster.Mean[m].x-Cluster.DataMinX)*
				  Cluster.DataScaleX-4),
		   (unsigned int)((Cluster.Mean[m].y-Cluster.DataMinY)*
				  Cluster.DataScaleY-4),
		   8,8);
    sprintf(str,"%d",m);//ddd
    XDrawString(display,pixmap,gc,(unsigned int)Cluster.Mean[m].x-2,
		(unsigned int)Cluster.Mean[m].y-2,str,strlen(str));//ddd
  }
  // Call draw (normally call after an expose event was created)
  // with a face event.
  Fake_draw();
}

void TXClusterGUI::draw_done() {
  set_Fg_color(); XDrawString(display,pixmap,gc,5,20,"Done",4);
  Fake_draw(); 
}

void TXClusterGUI::Fake_draw() {
  XExposeEvent fake_event;
  fake_event.width = pixmapWidth; 
  fake_event.height = pixmapHeight; 
  draw(drawingArea,(XEvent *)&fake_event,NULL,NULL);
}

void TXClusterGUI::draw(Widget w, XEvent *event, 
		      String *args, Cardinal *num_args) {
  XExposeEvent *exposeEvent = (XExposeEvent *) event;
  GC gc;
  Dimension Width, Height;
  //Dimension ExRegWidth, ExRegHeight;
  //ExRegWidth = exposeEvent->width; ExRegHeight = exposeEvent->height;
  Width = exposeEvent->width; Height = exposeEvent->height;
  //cout<<"Width:"<<Width<<" Height:"<<Height<<endl;//ddd
  if (!XtIsRealized(w)) return; 
  XtVaGetValues(w, XmNuserData, &gc, NULL);
  //XtVaGetValues(w, XmNwidth, &Width, NULL);
  //XtVaGetValues(w, XmNheight, &Height, NULL);
  XCopyArea(display, pixmap, XtWindow(w), 
	    gc, 0, 0, Width, Height, 0, 0);
  //cout << "draw\n";
  //cout << "width:" << Width << " height:" << Height << endl;
  //cout << "ExRegWidth:" << ExRegWidth 
  //     << " ExRegHeight:" << ExRegHeight << endl;
}

void TXClusterGUI::resize(Widget w, XEvent *event, 
			String *args, Cardinal *num_args) {
  XConfigureEvent *configEvent = (XConfigureEvent *) event;
  GC gc;
  Dimension newWidth, newHeight;
  newWidth = configEvent->width; newHeight = configEvent->height;
  XtVaGetValues(w,XmNuserData, &gc, NULL);
  //XtVaGetValues(w,XmNwidth,&newWidth,NULL);
  //XtVaGetValues(w,XmNheight,&newHeight,NULL);
  cout << "resize -- newwidth:" << newWidth 
       << " newheight:" << newHeight << endl;
  cout << "pixmapWidth:" << pixmapWidth 
       << " pixmapHeight:" << pixmapHeight << endl;
  //  resize does nothing unless new size is bigger than entire pixmap
  if( (newWidth <= pixmapWidth) && (newHeight <= pixmapHeight)) return;
  // Reallocate a bigger pixmap.
  Pixmap tmpPixmap = XCreatePixmap
    (XtDisplay(w), RootWindowOfScreen(XtScreen(w)), newWidth, newHeight,
     DefaultDepthOfScreen(XtScreen(w)));
  XCopyArea(display, pixmap, tmpPixmap,
	    gc ,0, 0, pixmapWidth, pixmapHeight, 0, 0);
  XFreePixmap(display, pixmap);
  pixmap = tmpPixmap; pixmapWidth=newWidth; pixmapHeight=newHeight;
}

void TXClusterGUI::wait(double seconds) {
  time_t now = time(NULL); while(difftime(time(NULL),now)<seconds); 
}

void TXClusterGUI::set_color_pixels() {
  XColor col, exactColor;
  if(!ColPix) ColPix = new (long unsigned int)[NbColors];
  else XFreeColors(display, cmap, ColPix, NbColors, 0);
  for(unsigned int c=0;c<NbColors;c++) {
    if(!XAllocNamedColor(display, cmap, Colors[c], &col, &exactColor)) {
      char buf[32]; sprintf(buf, "Can't alloc %s", Colors[c]);//ddd
      ColPix[c] = set_Fg_color();
    } else ColPix[c] = col.pixel;
  }
}

void TXClusterGUI::set_color(long unsigned int pix) {
  XSetForeground(display, gc, ColPix[pix]); 
}

void TXClusterGUI::set_color(String color) {
  XColor col, exactColor;
  if(!XAllocNamedColor(display, cmap, color, &col, &exactColor)) {
    char buf[32]; sprintf(buf, "Can't alloc %s", color);//ddd
    //XtWarning(buf);
    set_Fg_color();
    return;
  }
  XSetForeground(display, gc, col.pixel);
}

long unsigned int TXClusterGUI::set_Fg_color() {
  long unsigned int pix = BlackPixelOfScreen(XtScreen(drawingArea));
  XSetForeground(display, gc, pix);
  return pix;
}

long unsigned int TXClusterGUI::set_Bg_color() {
  long unsigned int pix = WhitePixelOfScreen(XtScreen(drawingArea));
  XSetForeground(display, gc, pix);
  return pix;
}
 
char TXClusterGUI::LoadFont(char *AFontName, GC gc)
{
  char TempFontName[20];
  strcpy(TempFontName, AFontName);
  if (!strcmp(AFontName, "default")) 
    strcpy(TempFontName, "8x13");
  if((fontStruct) && (!strcmp(fontName, TempFontName))) return 0;
  else
    if(fontStruct) XFreeFont(display, fontStruct);
  /* get the font, the font ID */
  fontStruct = XLoadQueryFont(display, TempFontName);
  if (fontStruct == NULL) {
      fprintf(stderr, "Can't XLoadQueryFont !\n");
      return 1;
  }
  XSetFont(display, gc, fontStruct->fid);
  strcpy(fontName, TempFontName);
  return (0);
}

Boolean TXClusterGUI::run_k_means(XtPointer client_data) {
  //cout << "run_algo" << endl;
  bool finished = Cluster.k_means();
  draw_algo();
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_Pairing(XtPointer client_data) {
  bool finished = Cluster.Pairing();
  if(!Cluster.DisplayOnlyResult || finished) draw_algo(); 
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_Grouping(XtPointer client_data) {
  bool finished = Cluster.Grouping();
  if(!Cluster.DisplayOnlyResult || finished) draw_algo();
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_Neting(XtPointer client_data) {
  bool finished = Cluster.Neting();
  if(!Cluster.DisplayOnlyResult || finished) draw_algo();
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_CNeting(XtPointer client_data) {
  bool finished = Cluster.CNeting();
  if(!Cluster.DisplayOnlyResult || finished) draw_algo(); 
  if(Cluster.WaitSeconds) wait(Cluster.WaitSeconds);
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_RCNeting(XtPointer client_data) {
  bool finished = Cluster.RCNeting();
  if((!Cluster.DisplayOnlyResult && Cluster.changed) || finished) {
    draw_algo(); 
    //if(Cluster.WaitSeconds) wait(Cluster.WaitSeconds);
  }
  if(finished) { draw_done(); return true; }
  return false;
}

Boolean TXClusterGUI::run_random(XtPointer client_data) {
  cout << "run_random" << endl;
  return true;
}

//// main
int main(int argc, char** argv) {
  TXClusterGUI XClusterGUI;
  XClusterGUI.Init(argc, argv);
}
