// XCluster.h
// All variables that the GUI uses are static because the 
// the callback functions have to be static.
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <unistd.h> // sleep.
#include <time.h> // wait.
#include <iostream.h>
#include <Xm/Xm.h>// Standart motif includes.
#include <Xm/RepType.h>
#include <Xm/MainW.h> // MainWindow
#include <Xm/Frame.h> 
#include <Xm/Label.h> 
#include <Xm/Text.h>      // XmText widget.
#include <Xm/RowColumn.h> // MenuBar
#include <Xm/SeparatoG.h> // Seperator in menu.
#include <Xm/MessageB.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/ToggleBG.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawingA.h>  // drawing area
// #include <Xw/WorkSpace.h> // canvas
#include <Xm/Form.h>      // pixmap box 
#include <Xm/BulletinB.h> // XmBulletinBoard  
#include "PatternManagement.h"


////TCluster
class TCluster : public TPatternGenerator {
public:
  TCluster();
  ~TCluster();
  void DeleteAllExceptData();
  void NewData();
  void NewMean();
  char LoadData();//XCluster.dat:Two values in one row 2xlinebreak ends data.
  char SaveData();//XCluster.dat:Two values in one row 2xlinebreak ends data.
  void GenerateData(String DataName);
  void InitData();// To init drawing variables for data.
  void Init(String AlgoName);
  bool k_means();
  bool Pairing();
  bool Grouping();
  bool Neting();
  bool CNeting();// Coutinual Neting.
  bool RCNeting();// Regularized Coutinual Neting.
  double DistPointPoint(unsigned int i, unsigned int ii);
  double DistPointMean(unsigned int i, unsigned int m);
  double DistMeanMean(unsigned int m, unsigned int mm);
  void ReasignData();
  void AdjustStepSize();
  void MergeClusterGroups();
  void FillHolesInMean();
  // Functions to manage hierarchystep clustering solutions.
  void InitHierarchy(bool InitDataTagHierarchy, unsigned int aMaxHierarchies);
  void DeleteHierarchy();
  void StoreHierarchy();
  void RestoreHierarchy(unsigned int step);
  void SelectBestHierarchyStep(char* Criterum); // Also restore this step.
  void Init_DataDistMatrix();
  void Delete_DataDistMatrix();
  void Calc_DataDistMatrix();
  void Sort_DataDistMatrix(); // Each line vector according to dist.
  void Con_DataDistMatrix(); // Set Con(nections) of Hierarchystep.
  void Init_ConVec(); 
  void SwapInConVec(unsigned int DstPos, unsigned int SrcPos); 
  void ReSort_ConVec(); 
  void AddToCluster(unsigned int ClusterNb, unsigned int receiver);
  unsigned int DataSize; // Number of points
  unsigned int DataAreaX, DataAreaY; // Drawing area in points.
  struct TData { unsigned int tag,Px,Py; // Pxy for darwing.
    double x,y,dx,dy,dist,weight; 
    bool changed; } *Data;
  double DataMinX, DataMinY, DataMaxX, DataMaxY, DataRangeX, DataRangeY, 
    DataScaleX, DataScaleY;
  unsigned int maxLoops;
  unsigned int NbClusters;
  // TMean.C:number of points in Cluster.
  struct TMean { double x,y,dx,dy,dist; unsigned int C,nN; bool changed; 
  } *Mean,**MeanHierarchy;
  // HierarchyStep of possible clustering solutions.
  unsigned int MaxHierarchies, HierarchyStep, *NbClustersHierarchy;
  // Corresponding to a HierarchyStep, to reconstract data tags.
  unsigned int **DataTagHierarchy; 
  // Varibles for statistics and control.
  unsigned int loop; 
  bool changed; // For stopping criteria.
  unsigned int NbDistinctClusters;

  // Variables for display.
  double WaitSeconds; 
  char DisplayOnlyResult;

  // Variables for Pairing.
  double BaseStepSize;
  double Resolution;
  double StepSize;

  // Variables for neting, Con relates to unsorted matrix!.
  //To evaluate and cut off outliers.
  double ConMean, ConStdDev, ConErr, ConCutOffDist;
  double ConCutOffFactor;
  struct TNeighbor { unsigned int Nb,Con; double dist; };
  static int CompareDist(const void *vN1, const void *vN2) { 
      TNeighbor* N1 = (TNeighbor*)vN1; 
      TNeighbor* N2 = (TNeighbor*)vN2; 
      // Used to qsort DataDistMatrix.
      //static int CompareDist(const void *N1, const void *N2);
      //if(((TNeighbor*)N1)->dist<((TNeighbor*)N2)->dist) return -1; 
      //else if(((TNeighbor*)N1)->dist>((TNeighbor*)N2)->dist) return 1; 
      if(N1->dist<N2->dist) return -1;
      else if(N1->dist>N2->dist) return 1; 
      return 0; }
  TNeighbor **DataDistMatrix;

  // Vector of connections (used for CNeting).
  struct TConVec { unsigned int SRC,DST,NbClusters; double dist; } *ConVec; 
  unsigned int ConVecSize, ConVecPos;
  static int CompareDistInConVec(const void *vC1, const void *vC2) { 
      // qsort ConVec.
      TConVec* C1 = (TConVec*)vC1; 
      TConVec* C2 = (TConVec*)vC2; 
      if(C1->dist<C2->dist)return -1; 
      else if(C1->dist>C2->dist)return 1; return 0;}
};     


//// TXClusterGUI
class TXClusterGUI {
public:
  TXClusterGUI();
  ~TXClusterGUI();
  void Init(int argc, char** argv);
  char InitApp(int argc, char** argv);
  void InitActions();
  void InitMenu();
  void InitPopupDialoges();
  void InitGraphics();
  static void wait(double seconds);
  static void set_color_pixels();
  static void set_color(long unsigned int pix);
  static void set_color(String color);
  static long unsigned int set_Fg_color();
  static long unsigned int set_Bg_color();
  static char LoadFont(char *AFontName, GC gc);
  // Action (Callback) functions.
  static void File_Quit(Widget, XEvent *,String *, Cardinal *);
  static void File_Start(Widget, XEvent *,String *, Cardinal *);
  static void File_Cont(Widget, XEvent *,String *, Cardinal *);
  static void File_Stop(Widget, XEvent *,String *, Cardinal *);
  static void File_Load_Data(Widget, XEvent *,String *, Cardinal *);
  static void File_Save_Data(Widget, XEvent *,String *, Cardinal *);
  static void File_Generate_Data(Widget, XEvent *,String *, Cardinal *);
  static void Algo_toggle_cbk(Widget, XtPointer, XtPointer);
  static void Algo_Parameter(Widget, XEvent *,String *, Cardinal *);
  static void PopdownDialogBox_cbk(Widget, XtPointer, XtPointer);
  static void Fake_draw();
  static void draw(Widget, XEvent *,String *, Cardinal *);
  static void resize(Widget, XEvent *,String *, Cardinal *);
  static void draw_algo();
  static void draw_done();
  static Boolean run_k_means(XtPointer);
  static Boolean run_Pairing(XtPointer);
  static Boolean run_Grouping(XtPointer);
  static Boolean run_Neting(XtPointer);
  static Boolean run_CNeting(XtPointer);
  static Boolean run_RCNeting(XtPointer);
  static Boolean run_random(XtPointer);
  // GUI variables
  int WIDTH;
  int HEIGHT;
  static XtAppContext app_context;
  static Widget topLevel;
  static Widget mainWindow;
  static Widget k_means_TogBut;
  static Widget Pairing_TogBut, Grouping_TogBut, Neting_TogBut, 
    CNeting_TogBut, RCNeting_TogBut;
  static Widget random_TogBut; 
  static Widget dialog_shell;
  static Widget dialog_box, dialog_label, dialog_text, dialog_btn;
  static Display *display;
  static Window window;
  static Widget drawingArea;
  static Pixmap pixmap; 
  static Dimension pixmapWidth;
  static Dimension pixmapHeight;
  static Colormap cmap;
  static GC gc; 
  XGCValues gc_values; XtGCMask gc_mask;
  static const unsigned int NbColors = 20;
  static const String Colors[NbColors];
  static long unsigned int *ColPix;
  static XFontStruct *fontStruct;
  static char fontName[];

  // Variables for event handling.
  static XtWorkProcId workProcId;
  static XtWorkProc algo_workProc;
  static String AlgoName;

  // Non GUI vaiarbles
  static TCluster Cluster;
};


Display* TXClusterGUI::display;
Window TXClusterGUI::window;
Widget TXClusterGUI::topLevel;
Widget TXClusterGUI::mainWindow;
Widget TXClusterGUI::drawingArea;
Pixmap TXClusterGUI::pixmap;
Dimension TXClusterGUI::pixmapWidth;
Dimension TXClusterGUI::pixmapHeight;
Colormap TXClusterGUI::cmap;
GC TXClusterGUI::gc; 
TCluster TXClusterGUI::Cluster;
const unsigned int TXClusterGUI::NbColors;
const String TXClusterGUI::Colors[TXClusterGUI::NbColors] = { 
  "black","red","blue","green","cyan",
  "yellow","magenta","orange","purple","maroon",
  "khaki","navy","sienna","peru","beige",
  "pink","gold","brown","tomato","DeepPink"};
long unsigned int *TXClusterGUI::ColPix;
XFontStruct *TXClusterGUI::fontStruct;
char TXClusterGUI::fontName[] = "8x13";; 
XtWorkProcId TXClusterGUI::workProcId;
XtAppContext TXClusterGUI::app_context;
XtWorkProc TXClusterGUI::algo_workProc;
String TXClusterGUI::AlgoName;
Widget TXClusterGUI::k_means_TogBut;
Widget TXClusterGUI::Pairing_TogBut; 
Widget TXClusterGUI::Grouping_TogBut; 
Widget TXClusterGUI::Neting_TogBut; 
Widget TXClusterGUI::CNeting_TogBut; 
Widget TXClusterGUI::RCNeting_TogBut; 
Widget TXClusterGUI::random_TogBut; 
Widget TXClusterGUI::dialog_shell;
Widget TXClusterGUI::dialog_box;
Widget TXClusterGUI::dialog_label;
Widget TXClusterGUI::dialog_text;
Widget TXClusterGUI::dialog_btn;
